@@ -323,10 +323,10 @@ static int mysql_drv_real_connect(db_mysql_conn_t *db_mysql_con)
 
     mysql_ssl_set(con, ssl_key, ssl_cert, ssl_ca, NULL, args.ssl_cipher);
 
-#ifdef MYSQL_OPT_SSL_MODE
+#ifdef HAVE_MYSQL_OPT_SSL_MODE
     unsigned int opt_ssl_mode = SSL_MODE_REQUIRED;
 
-    DEBUG("mysql_options(%p,%s,%u)", con, "MYSQL_OPT_SSL_MODE", opt_ssl_mode);
+    DEBUG("mysql_options(%p, %s, %u)", con, "MYSQL_OPT_SSL_MODE", opt_ssl_mode);
     mysql_options(con, MYSQL_OPT_SSL_MODE, &opt_ssl_mode);
 #endif
   }
@@ -969,8 +969,10 @@ int mysql_drv_fetch_row(db_result_t *rs, db_row_t *row)
   DEBUG("mysql_fetch_row(%p) = %p", rs->ptr, my_row);
 
   unsigned long *lengths = mysql_fetch_lengths(rs->ptr);
+  DEBUG("mysql_fetch_lengths(%p) = %p", rs->ptr, lengths);
+
   if (lengths == NULL)
-    return DB_ERROR_NONE;
+    return DB_ERROR_IGNORABLE;
 
   for (size_t i = 0; i < rs->nfields; i++)
   {
